@@ -13,12 +13,16 @@ pub use release::*;
 
 use std::{
     env::{self, args},
+    ffi::OsStr,
     process::Stdio,
 };
 
 #[cfg(unix)]
 use std::{fs::File, os::unix::fs::PermissionsExt};
 
+#[cfg(windows)]
+use self_replace;
+
 use axoasset::LocalAsset;
 use axoprocess::Cmd;
 pub use axotag::Version;
@@ -26,6 +30,9 @@ use camino::Utf8PathBuf;
 
 use tempfile::TempDir;
 
+/// Version number for this release of axoupdater.
+pub const VERSION: &str = env!("CARGO_PKG_VERSION");
+
 /// Provides information about the result of the upgrade operation
 pub struct UpdateResult {
     /// The old version (pre-upgrade)
@@ -97,6 +104,8 @@ pub struct AxoUpdater {
     /// When set to true, skips performing version checks and always assumes
     /// the software is out of date.
     always_update: bool,
+    /// Whether to modify the system path when installing
+    modify_path: bool,
 }
 
 impl Default for AxoUpdater {
@@ -123,6 +132,7 @@ impl AxoUpdater {
             installer_path: None,
             tokens: AuthorizationTokens::default(),
             always_update: false,
+            modify_path: true,
         }
     }
 
@@ -141,6 +151,7 @@ impl AxoUpdater {
             installer_path: None,
             tokens: AuthorizationTokens::default(),
             always_update: false,
+            modify_path: true,
         }
     }
 
@@ -170,6 +181,7 @@ impl AxoUpdater {
             installer_path: None,
             tokens: AuthorizationTokens::default(),
             always_update: false,
+            modify_path: true,
         })
     }
 
@@ -190,6 +202,19 @@ impl AxoUpdater {
         Ok(self)
     }
 
+    /// Changes this updater's name to `app_name`, regardless of what it was
+    /// initialized as and regardless of what was read from the receipt.
+    pub fn set_name(&mut self, app_name: &str) -> &mut AxoUpdater {
+        self.name = Some(app_name.to_owned());
+        if let Some(source) = &self.source {
+            let mut our_source = source.clone();
+            our_source.app_name = app_name.to_owned();
+            self.source = Some(our_source);
+        }
+
+        self
+    }
+
     /// Enables printing the underlying installer's stdout.
     pub fn enable_installer_stdout(&mut self) -> &mut AxoUpdater {
         self.print_installer_stdout = true;
@@ -339,7 +364,7 @@ impl AxoUpdater {
     /// component if necessary. Works around a bug introduced in cargo-dist
     /// where this field was returned inconsistently in receipts for a few
     /// versions.
-    fn install_prefix_root(&self) -> AxoupdateResult<Utf8PathBuf> {
+    pub fn install_prefix_root(&self) -> AxoupdateResult<Utf8PathBuf> {
         let Some(install_prefix) = &self.install_prefix else {
             return Err(AxoupdateError::NotConfigured {
                 missing_field: "install_prefix".to_owned(),
@@ -453,34 +478,20 @@ impl AxoUpdater {
             installer_path
         };
 
-        // Before we update, move ourselves to a temporary directory.
+        // Before we update, rename ourselves to a temporary name.
         // This is necessary because Windows won't let an actively-running
         // executable be overwritten.
         // If the update fails, we'll move it back to where it was before
         // we began the update process.
-        // NOTE: this TempDir needs to be held alive for the whole function.
-        let temp_root;
         let to_restore = if cfg!(target_family = "windows") {
-            // If we know the install prefix, place the temporary directory for
-            // the executable there. This is because the `rename` syscall can't
-            // rename an executable across filesystem bounds, and there's a
-            // chance the system temporary directory could be on a different
-            // drive than the executable is.
-            // (A copy-and-delete move won't work because Windows won't let us
-            // delete a running executable - the same reason we're moving it out
-            // of the way to begin with!)
-            // See https://github.com/axodotdev/axoupdater/issues/120.
-            temp_root = if let Ok(prefix) = self.install_prefix_root() {
-                TempDir::new_in(prefix)?
-            } else {
-                TempDir::new()?
-            };
-            let old_path = std::env::current_exe()?;
-            let old_filename = old_path.file_name().expect("current binary has no name!?");
-            let ourselves = temp_root.path().join(old_filename);
-            std::fs::rename(&old_path, &ourselves)?;
+            let old_filename = std::env::current_exe()?;
 
-            Some((ourselves, old_path))
+            let mut new_filename = old_filename.as_os_str().to_os_string();
+            // Filename follows the pattern set here: https://docs.rs/self-replace/1.5.0/self_replace/#implementation
+            new_filename.push(OsStr::new(".previous.exe"));
+            std::fs::rename(&old_filename, &new_filename)?;
+
+            Some((new_filename, old_filename))
         } else {
             None
         };
@@ -517,9 +528,16 @@ impl AxoUpdater {
         // Also set the app-specific name for this; in the future, the
         // CARGO_DIST_ version may be removed.
         let app_name = self.name.clone().unwrap_or_default();
-        let app_specific_env_var = app_name.to_ascii_uppercase().replace('-', "_") + "_INSTALL_DIR";
+        let app_name_env_var = app_name_to_env_var(&app_name);
+        let app_specific_env_var = format!("{app_name_env_var}_INSTALL_DIR");
         command.env(app_specific_env_var, &install_prefix);
 
+        // If the previous installation didn't modify the path, we shouldn't either
+        if !self.modify_path {
+            let app_specific_modify_path = format!("{app_name_env_var}_NO_MODIFY_PATH");
+            command.env(app_specific_modify_path, "1");
+        }
+
         let result = command.output();
 
         let failed;
@@ -546,9 +564,13 @@ impl AxoUpdater {
             statuscode = None;
         }
 
-        if failed {
-            if let Some((ourselves, old_path)) = to_restore {
+        if let Some((ourselves, old_path)) = to_restore {
+            if failed {
                 std::fs::rename(ourselves, old_path)?;
+            } else {
+                #[cfg(windows)]
+                self_replace::self_delete_at(&ourselves)
+                    .map_err(|_| AxoupdateError::CleanupFailed {})?;
             }
         }
 
@@ -615,6 +637,11 @@ fn get_app_name() -> Option<String> {
     }
 }
 
+/// Returns an environment variable-compatible version of the app name.
+pub fn app_name_to_env_var(app_name: &str) -> String {
+    app_name.to_ascii_uppercase().replace('-', "_")
+}
+
 fn root_without_bin(path: &Utf8PathBuf) -> Utf8PathBuf {
     if path.file_name() == Some("bin") {
         if let Some(parent) = path.parent() {
