@@ -9,10 +9,12 @@ use alloc::borrow::Cow;
 use alloc::string::String;
 use alloc::vec::Vec;
 use log::warn;
+use resource::ResourceData;
 
 pub mod authenticode;
 pub mod certificate_table;
 pub mod characteristic;
+pub mod clr;
 pub mod data_directories;
 pub mod debug;
 pub mod dll_characteristic;
@@ -20,9 +22,11 @@ pub mod exception;
 pub mod export;
 pub mod header;
 pub mod import;
+pub mod load_config;
 pub mod optional_header;
 pub mod options;
 pub mod relocation;
+pub mod resource;
 pub mod section_table;
 pub mod subsystem;
 pub mod symbol;
@@ -31,6 +35,7 @@ pub mod utils;
 
 use crate::container;
 use crate::error;
+use crate::options::Permissive;
 use crate::pe::utils::pad;
 use crate::strtab;
 use options::ParseMode;
@@ -79,8 +84,14 @@ pub struct PE<'a> {
     pub exception_data: Option<exception::ExceptionData<'a>>,
     /// Base relocation data if any
     pub relocation_data: Option<relocation::RelocationData<'a>>,
+    /// Load config data if any
+    pub load_config_data: Option<load_config::LoadConfigData>,
     /// Certificates present, if any, described by the Certificate Table
     pub certificates: certificate_table::CertificateDirectoryTable<'a>,
+    /// Resource information if any
+    pub resource_data: Option<ResourceData<'a>>,
+    /// CLR managed data if present
+    pub clr_data: Option<clr::ClrData<'a>>,
 }
 
 impl<'a> PE<'a> {
@@ -91,7 +102,7 @@ impl<'a> PE<'a> {
 
     /// Reads a PE binary from the underlying `bytes`
     pub fn parse_with_opts(bytes: &'a [u8], opts: &options::ParseOptions) -> error::Result<Self> {
-        let header = header::Header::parse(bytes)?;
+        let header = header::Header::parse_with_opts(bytes, opts)?;
         let mut authenticode_excluded_sections = None;
 
         debug!("{:#?}", header);
@@ -101,7 +112,7 @@ impl<'a> PE<'a> {
         let offset =
             &mut (optional_header_offset + header.coff_header.size_of_optional_header as usize);
 
-        let sections = header.coff_header.sections(bytes, offset)?;
+        let sections = header.coff_header.sections_with_opts(bytes, offset, opts)?;
         let is_lib = characteristic::is_dll(header.coff_header.characteristics);
         let mut entry = 0;
         let mut image_base = 0;
@@ -115,7 +126,10 @@ impl<'a> PE<'a> {
         let mut tls_data = None;
         let mut exception_data = None;
         let mut relocation_data = None;
+        let mut load_config_data = None;
         let mut certificates = Default::default();
+        let mut resource_data = Default::default();
+        let mut clr_data = Default::default();
         let mut is_64 = false;
         if let Some(optional_header) = header.optional_header {
             // Sections we are assembling through the parsing, eventually, it will be passed
@@ -215,13 +229,18 @@ impl<'a> PE<'a> {
             }
             debug!("imports: {:#?}", imports);
             if let Some(&debug_table) = optional_header.data_directories.get_debug_table() {
-                debug_data = Some(debug::DebugData::parse_with_opts(
+                debug_data = debug::DebugData::parse_with_opts(
                     bytes,
                     debug_table,
                     &sections,
                     file_alignment,
                     opts,
-                )?);
+                )
+                .map(Some)
+                .or_permissive_and_default(
+                    opts.parse_mode.is_permissive(),
+                    "Failed to parse resource data",
+                )?;
             }
 
             if let Some(tls_table) = optional_header.data_directories.get_tls_table() {
@@ -233,7 +252,13 @@ impl<'a> PE<'a> {
                     file_alignment,
                     opts,
                     is_64,
-                )?;
+                ) // Result<Option<T>>
+                .or_else(|e| {
+                    opts.parse_mode
+                        .is_permissive()
+                        .then_some(None) // Permissive=true -> Some(None)
+                        .ok_or(e) // Some(None) -> Ok(None), None -> Err(e)
+                })?;
                 debug!("tls data: {:#?}", tls_data);
             }
 
@@ -243,26 +268,73 @@ impl<'a> PE<'a> {
                 if let Some(&exception_table) =
                     optional_header.data_directories.get_exception_table()
                 {
-                    exception_data = Some(exception::ExceptionData::parse_with_opts(
+                    exception_data = exception::ExceptionData::parse_with_opts(
                         bytes,
                         exception_table,
                         &sections,
                         file_alignment,
                         opts,
-                    )?);
+                    )
+                    .map(Some)
+                    .or_permissive_and_default(
+                        opts.parse_mode.is_permissive(),
+                        "Failed to parse security data",
+                    )?;
                 }
             }
 
             if let Some(&baserelocs_dir) =
                 optional_header.data_directories.get_base_relocation_table()
             {
-                relocation_data = Some(relocation::RelocationData::parse_with_opts(
+                relocation_data = relocation::RelocationData::parse_with_opts(
                     bytes,
                     baserelocs_dir,
                     &sections,
                     file_alignment,
                     opts,
-                )?);
+                )
+                .map(Some)
+                .or_permissive_and_default(
+                    opts.parse_mode.is_permissive(),
+                    "Failed to parse base relocation data",
+                )?;
+            }
+
+            if let Some(&load_config_dir) = optional_header.data_directories.get_load_config_table()
+            {
+                debug!(
+                    "LoadConfig directory found: virtual_address={:#x}, size={:#x}",
+                    load_config_dir.virtual_address, load_config_dir.size
+                );
+                load_config_data = load_config::LoadConfigData::parse_with_opts(
+                    bytes,
+                    load_config_dir,
+                    &sections,
+                    file_alignment,
+                    opts,
+                    is_64,
+                )
+                .map(Some)
+                .or_permissive_and_default(
+                    opts.parse_mode.is_permissive(),
+                    "Failed to parse load config data",
+                )?;
+            }
+
+            if let Some(com_descriptor) = optional_header.data_directories.get_clr_runtime_header()
+            {
+                clr_data = clr::ClrData::parse_with_opts(
+                    bytes,
+                    &com_descriptor,
+                    &sections,
+                    file_alignment,
+                    opts,
+                )
+                .map(Some)
+                .or_permissive_and_default(
+                    opts.parse_mode.is_permissive(),
+                    "Failed to parse CLR runtime data",
+                )?;
             }
 
             // Parse attribute certificates unless opted out of
@@ -292,6 +364,18 @@ impl<'a> PE<'a> {
                 0
             };
 
+            if let Some(&resource_table) = optional_header.data_directories.get_resource_table() {
+                let data = resource::ResourceData::parse_with_opts(
+                    bytes,
+                    resource_table,
+                    &sections,
+                    file_alignment,
+                    opts,
+                )?;
+                resource_data = Some(data);
+                debug!("resource_data data: {:#?}", data.version_info);
+            }
+
             authenticode_excluded_sections = Some(authenticode::ExcludedSections::new(
                 checksum,
                 datadir_entry_certtable,
@@ -319,7 +403,10 @@ impl<'a> PE<'a> {
             tls_data,
             exception_data,
             relocation_data,
+            load_config_data,
             certificates,
+            resource_data,
+            clr_data,
         })
     }
 
@@ -605,8 +692,8 @@ impl<'a> Coff<'a> {
 
 #[cfg(test)]
 mod tests {
-    use super::Coff;
-    use super::PE;
+    use super::*;
+    use crate::pe::options::{ParseMode, ParseOptions};
 
     static INVALID_DOS_SIGNATURE: [u8; 512] = [
         0x3D, 0x5A, 0x90, 0x00, 0x03, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00,
@@ -794,4 +881,46 @@ mod tests {
         assert_eq!(binary.entry, 0x1020u32);
         assert_eq!(binary.image_base, 0x140000000u64);
     }
+
+    #[test]
+    fn test_packed_binary_permissive_parsing() {
+        // Create a minimal PE with a PE pointer that's before the DOS stub offset
+        // This simulates a packed binary scenario
+        let mut packed_pe = vec![0u8; 0x100];
+
+        // DOS header with MZ signature
+        packed_pe[0] = 0x4D; // 'M'
+        packed_pe[1] = 0x5A; // 'Z'
+
+        // Set PE pointer to 0x30 (before DOS_STUB_OFFSET which is 0x40)
+        packed_pe[0x3C] = 0x30;
+        packed_pe[0x3D] = 0x00;
+        packed_pe[0x3E] = 0x00;
+        packed_pe[0x3F] = 0x00;
+
+        // PE signature at offset 0x30
+        packed_pe[0x30] = 0x50; // 'P'
+        packed_pe[0x31] = 0x45; // 'E'
+        packed_pe[0x32] = 0x00;
+        packed_pe[0x33] = 0x00;
+
+        // Minimal COFF header (machine type, etc.)
+        packed_pe[0x34] = 0x4C; // IMAGE_FILE_MACHINE_I386
+        packed_pe[0x35] = 0x01;
+
+        // Test strict parsing - should fail
+        let strict_result = PE::parse(&packed_pe);
+        assert!(
+            strict_result.is_err(),
+            "Strict parsing should fail for packed binary"
+        );
+
+        // Test permissive parsing - should succeed
+        let permissive_opts = ParseOptions::default().with_parse_mode(ParseMode::Permissive);
+        let permissive_result = PE::parse_with_opts(&packed_pe, &permissive_opts);
+        assert!(
+            permissive_result.is_ok(),
+            "Permissive parsing should succeed for packed binary"
+        );
+    }
 }
